SystemOrganization addCategory: 'ChronosSystem-Gemstone'!

!Dictionary methodsFor: '*chronossystem-gemstone' stamp: 'PaulDeBruicker 03/06/2011 18:27'!
declare: key from: aDictionary 
	"Add key to the receiver. If key already exists, do nothing. If aDictionary 
	includes key, then remove it from aDictionary and use its association as 
	the element of the receiver."

	(self includesKey: key) ifTrue: [^ self].
	(aDictionary includesKey: key)
		ifTrue: 
			[self add: (aDictionary associationAt: key).
			aDictionary removeKey: key]
		ifFalse: 
			[self add: key -> nil]! !

!Number methodsFor: '*chronossystem-gemstone' stamp: 'PaulDeBruicker 3/9/2011 14:50'!
year
	^Duration days: self * 365! !

!Number methodsFor: '*chronossystem-gemstone' stamp: 'PaulDeBruicker 3/9/2011 14:51'!
years
	^CalendarDuration days: (self * 365)! !

!GeographicCoordinateSpec class methodsFor: '*chronossystem-gemstone' stamp: 'pad 03/07/2011 16:36'!
readFromString: aString 
	"Create an object based on the contents of aString."
	^ self readFrom: aString readStream! !

ChronosEnvironment subclass: #ChronosGemstoneEnvironment
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: 'Chronos ChronosConstants Passport'
	category: 'ChronosSystem-Gemstone'!

!ChronosGemstoneEnvironment classSide methodsFor: 'accessing' stamp: 'PaulDeBruicker 03/06/2011 16:29'!
chronosNamespace
	"Answer the namespace that contains the classes of the Chronos codebase."

	^self globalNamespace at: self chronosNamespaceName! !

!ChronosGemstoneEnvironment classSide methodsFor: 'accessing' stamp: 'PaulDeBruicker 03/09/2011 19:59'!
defineChronosNamespaces
	"self defineChronosNamespace.
	self defineChronosTemporalRatioConstantsNamespace.
	self defineChronosDaysOfWeekConstantsNamespace.
	self defineChronosStandardMonthConstantsNamespace.
	self defineChronosFactoryBindingsNamespace.
	self defineChronosCalendarConstantsNamespace.
	self defineChronosDurationConstantsNamespace.
	self defineChronosConstantsNamespace."! !

!ChronosGemstoneEnvironment classSide methodsFor: 'accessing' stamp: 'PaulDeBruicker 03/09/2011 19:59'!
defineChronosSharedAndGlobalVariables
"Most of this is taken care of in the ChronosConstants class initialization method.  "
	self defineChronosGlobalsAndAliases! !

!ChronosGemstoneEnvironment classSide methodsFor: 'private' stamp: 'PaulDeBruicker 03/06/2011 16:31'!
setPlatformClass
	PlatformClass := ChronosGemstoneEnvironment! !

!ChronosGemstoneEnvironment classSide methodsFor: 'private' stamp: 'PaulDeBruicker 03/06/2011 16:29'!
setProtectedGlobalVariables
    ProtectedGlobalVariables := Set with: #ScaledDecimal with: #DateAndTime with: #Duration! !

!ChronosGemstoneEnvironment classSide methodsFor: 'accessing' stamp: 'PaulDeBruicker 03/06/2011 16:32'!
systemFacadeClass
	^ ChronosGemstoneSystemFacade ! !

!ChronosGemstoneEnvironment methodsFor: 'installation/setup' stamp: 'PaulDeBruicker 03/09/2011 20:38'!
initializeDurationConstants
	"ChronosEnvironment canonical initializeDurationConstants"


	ScientificDuration initializeClassVariables.
	CalendarDuration initializeClassVariables.
	(Smalltalk at: #ChronosConstants) assignChronosDurationConstants.
	! !

!ChronosGemstoneEnvironment methodsFor: 'installation/setup' stamp: 'PaulDeBruicker 03/09/2011 20:36'!
initializeFactoryBindings
	
	(Smalltalk at: #ChronosConstants) assignChronosFactoryBindings.
	! !

!ChronosGemstoneEnvironment methodsFor: 'installation/setup' stamp: 'PaulDeBruicker 03/09/2011 19:58'!
install


	super install.! !

!ChronosGemstoneEnvironment methodsFor: 'installation/setup' stamp: 'PaulDeBruicker 03/09/2011 20:37'!
setAnsiStandardCalendar: aGregorianCalendar
 (Smalltalk at: #ChronosConstants) setAnsiStandardCalendar: aGregorianCalendar   ! !

!ChronosGemstoneEnvironment methodsFor: 'installation/setup' stamp: 'PaulDeBruicker 03/09/2011 20:38'!
setAnsiStandardCalendarClock: aGregorianCalendarClock

	(Smalltalk at: #ChronosConstants) setAnsiStandardCalendarClock: aGregorianCalendarClock   ! !

!ChronosGemstoneEnvironment methodsFor: 'installation/setup' stamp: 'PaulDeBruicker 03/09/2011 20:37'!
setGregorian: aGregorianCalendar

	(Smalltalk at: #ChronosConstants) setGregorian: aGregorianCalendar   ! !

!ChronosGemstoneEnvironment methodsFor: 'installation/setup' stamp: 'PaulDeBruicker 03/09/2011 20:38'!
setJulian: aJulianCalendar

(Smalltalk at: #ChronosConstants) setJulian: aJulianCalendar   ! !

ChronosSystemClock subclass: #ChronosGemstoneSystemClock
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: 'Chronos ChronosConstants Passport'
	category: 'ChronosSystem-Gemstone'!

!ChronosGemstoneSystemClock methodsFor: 'testing' stamp: 'PaulDeBruicker 03/06/2011 16:29'!
isRelativeToUT
	"If the receiver reports ticks relative to UT, answer true.
	If the receiver reports ticks relative to local time, answer false."

	^false! !

!ChronosGemstoneSystemClock methodsFor: 'API-timing' stamp: 'PaulDeBruicker 03/06/2011 16:29'!
nanosecondsToRun: timedBlock
	^ChronosFunction times1000000: (Time millisecondsToRun: timedBlock)! !

!ChronosGemstoneSystemClock methodsFor: 'calendar clock binding' stamp: 'PaulDeBruicker 03/06/2011 16:29'!
secondsFromGregorianEpochUpToSystemClockEpoch
	"Answer the number of seconds from the Epoch date/time of the Proleptic Gregorian Calendar (i.e., January 1, 1 AD  00:00:00 Proleptic Gregorian) up to the Epoch date/time of the platform system clock encapsulated by the receiver (i.e., where clock ticks = 0).  For most Smalltallk implementations, the platform system clock is a software device provided by the virtual machine.  On most operating systems, the system clock's zero point will be 00:00:00 UT of the first day of some Gregorian year.  However, some platforms (e.g., Microsoft Windows) use a zero point that is 00:00:00 (midnight) LOCAL TIME.  Do not adjust the encapsulated clock's tick count from UT to local time, nor from local time to UT.  If the encapsulated clock reports ticks relative to UT, then #secondsFromGregorianEpochUpToSystemClockEpoch should answer the number of seconds since midnight UT of the clock's epoch date, and the instance method #isRelativeToUT should answer true. However, if the encapsulated clock reports ticks relative to local time, then #secondsFromGregorianEpochUpToSystemClockEpoch should answer the number of seconds since midnight local time of the clock's epoch date, and the instance method #isRelativeToUT should answer false.

	See the comment for the same method in ChronosSystemClock for help in porting to other systems, or for help in converting to/from foreign system clocks."

	"^Calendar gregorian secondsSinceEpochUpToYear: 1901" ^59958144000! !

!ChronosGemstoneSystemClock methodsFor: 'API-timing' stamp: 'PaulDeBruicker 03/06/2011 16:29'!
ticksNowSinceSystemClockEpoch
	"Answer the number of seconds since 00:00 on the morning of
	 January 1, 1901 (a 32-bit unsigned number).
	 Essential. See Object documentation whatIsAPrimitive. "

	^(DateAndTime now - DateAndTime epoch) asSeconds! !

ChronosSystemFacade subclass: #ChronosGemstoneSystemFacade
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: 'Chronos ChronosConstants Passport'
	category: 'ChronosSystem-Gemstone'!

!ChronosGemstoneSystemFacade methodsFor: 'as yet unclassified' stamp: 'PaulDeBruicker 03/09/2011 20:08'!
chronosDateAndTimeFromNativeCalendricalCoordinate: aNativeCalendricalCoordinate calendarClock: calendarClock 
	"Answer a Chronos DateAndTime that semantically corresponds as closely as possible to <aNativeTimestampOrDate>."

	"{ChronosSystemFacade current chronosDateAndTimeFromNativeCalendricalCoordinate: DateAndTime now calendarClock: Calendar gregorian clock}
	{ChronosSystemFacade current chronosDateAndTimeFromNativeCalendricalCoordinate: DateAndTime now calendarClock: Calendar julian clock}
	{ChronosSystemFacade current chronosDateAndTimeFromNativeCalendricalCoordinate: Date today calendarClock: Calendar julian clock}
	{ChronosSystemFacade current chronosDateAndTimeFromNativeCalendricalCoordinate: Date today calendarClock: Calendar gregorian clock}"

	| timepoint |
	
	timepoint := 
		DateAndTimeFactory basicNew
			setCalendar: calendarClock calendar;
			setJulianDayNumber: aNativeCalendricalCoordinate julianDayNumber.
 	^((aNativeCalendricalCoordinate respondsTo: #nanoSecond) 
		ifTrue: 
			[timepoint
				basicSetTimeZone: (self chronosDurationFromNativeDuration: aNativeCalendricalCoordinate offset) asTimezone;
				setHour: aNativeCalendricalCoordinate hour minute: aNativeCalendricalCoordinate minute second: aNativeCalendricalCoordinate second;
				setNanosecondsSinceSecond: aNativeCalendricalCoordinate nanoSecond]
		ifFalse: 
			[timepoint
				setSecondsSinceStartOfDay: 0;
				setNanosecondsSinceSecond: 0]) 
					canonicalizeFromLocalTime;
					beImmutable
! !

!ChronosGemstoneSystemFacade methodsFor: 'as yet unclassified' stamp: 'PaulDeBruicker 03/09/2011 20:08'!
chronosDateFromNativeCalendricalCoordinate: aNativeCalendricalCoordinate calendarClock: calendarClock 
	"Answer a Chronos date value that semantically corresponds as closely as possible to <aNativeTimestampOrDate>."

	"{ChronosSystemFacade current chronosDateFromNativeCalendricalCoordinate: DateAndTime now calendarClock: Calendar gregorian clock}
	{ChronosSystemFacade current chronosDateFromNativeCalendricalCoordinate: DateAndTime now calendarClock: Calendar julian clock}
	{ChronosSystemFacade current chronosDateFromNativeCalendricalCoordinate: Date today calendarClock: Calendar julian clock}
	{ChronosSystemFacade current chronosDateFromNativeCalendricalCoordinate: Date today calendarClock: Calendar gregorian clock}"

	| calendar |
	calendar := calendarClock calendar.
	^calendar isGregorian 
		ifTrue: 
			[DateFactory 
				year: aNativeCalendricalCoordinate year
				day: aNativeCalendricalCoordinate day]
		ifFalse: 
			[DateFactory  
				julianDay: aNativeCalendricalCoordinate julianDayNumber
				calendar: calendar]! !

!ChronosGemstoneSystemFacade methodsFor: 'as yet unclassified' stamp: 'PaulDeBruicker 03/09/2011 20:09'!
chronosDurationFromNativeDuration: aNativeDuration 
	"Answer a ScientificDuration that semantically corresponds as closely as possible to <aNativeDuration>."

	"ChronosSystemFacade current chronosDurationFromNativeDuration: (Duration hours: -8)"

	^DurationFactory seconds: aNativeDuration asSeconds nanoseconds: aNativeDuration nanoSeconds! !

!ChronosGemstoneSystemFacade methodsFor: 'as yet unclassified' stamp: 'PaulDeBruicker 03/09/2011 20:09'!
chronosTimeOfDayFromNativeTimeOfDay: aNativeTimeOfDay 
	"Answer a Chronos TimeOfDay that semantically corresponds as closely as possible to <aNativeTimeOfDay>."

	^TimeOfDayFactory 
		hour: aNativeTimeOfDay hours
		minute: aNativeTimeOfDay minutes
		second: aNativeTimeOfDay seconds! !

!ChronosGemstoneSystemFacade methodsFor: 'value conversion' stamp: 'PaulDeBruicker 03/09/2011 20:09'!
chronosTimezoneFromNativeTimeZone: aNativeTimeZone 
	"Answer a ChronosTimezone that semantically corresponds as closely as possible to <aNativeTimeZone>."
	"ChronosSystemFacade current chronosTimezoneFromNativeTimeZone: TimeZone default"

	| offsetSecondsFromUT |
	offsetSecondsFromUT := aNativeTimeZone offset asSeconds.
	^offsetSecondsFromUT = 0
		ifTrue: [ChronosTimezone universal]
		ifFalse: [((DurationFactory) seconds: offsetSecondsFromUT nanoseconds: aNativeTimeZone offset nanoSeconds) asTimezone]! !

!ChronosGemstoneSystemFacade methodsFor: 'private-system timezone' stamp: 'PaulDeBruicker 03/06/2011 16:30'!
defaultSystemTimeZoneBindingStrategy
	"Answer the strategy that Chronos should use by default whenever Chronos attempts resolve the Chronos system timezone based on the local timezone of the host system and/or Smalltalk execution environment. See class comment for a full explanation of the options.  This default can be overridden by sending control messages to the receiver."

	^#setSystemTimeZoneOnlyFromNativeSystemOlsonTimeZone

	"Options: 
		#setSystemTimeZoneToOlsonTimeZoneMostCongruentToNativeSystem
		#setSystemTimeZoneOnlyFromNativeSystemOlsonTimeZone 
		#setSystemTimeZoneToAnalogOfNativeSystemTimeZone"! !

!ChronosGemstoneSystemFacade methodsFor: 'testing' stamp: 'PaulDeBruicker 03/06/2011 16:30'!
isChronologyLibraryResident
	^Smalltalk includesKey: #TimeZone! !

!ChronosGemstoneSystemFacade methodsFor: 'value conversion' stamp: 'PaulDeBruicker 03/06/2011 16:30'!
nativeCalendricalCoordinateFromChronosDate: aChronosDate
	"Answer the native 'Date' analog (or 'Timestamp' analog, if no Date analog is available) that semantically corresponds as closely as possible to <aChronosDate>"

	"{ChronosSystemFacade current  nativeCalendricalCoordinateFromChronosDate: Gregorian clock dateToday}"
	"{ChronosSystemFacade current  nativeCalendricalCoordinateFromChronosDate: Julian clock dateToday}"

	^Date julianDayNumber: aChronosDate julianDayNumber! !

!ChronosGemstoneSystemFacade methodsFor: 'value conversion' stamp: 'PaulDeBruicker 03/06/2011 16:30'!
nativeCalendricalCoordinateFromChronosDateAndTime: aChronosDateAndTime 
	"Answer the native 'Timestamp' analog (or 'Date' analog, if no Timestamp analog is available) that semantically corresponds as closely as possible to <aChronosDateAndTime>"

	"{ChronosSystemFacade current  nativeCalendricalCoordinateFromChronosDateAndTime: Gregorian clock now}"
	"{ChronosSystemFacade current  nativeCalendricalCoordinateFromChronosDateAndTime: Julian clock now}"

	^DateAndTime
		year: aChronosDateAndTime year 
		month: aChronosDateAndTime month 
		day: aChronosDateAndTime dayOfMonth 
		hour: aChronosDateAndTime hour 
		minute: aChronosDateAndTime minute 
		second: aChronosDateAndTime second 
		nanoSecond: aChronosDateAndTime nanosecondsSinceSecond 
		offset: (self navitiveDurationFromChronosDuration: aChronosDateAndTime offset)! !

!ChronosGemstoneSystemFacade methodsFor: 'value conversion' stamp: 'PaulDeBruicker 03/06/2011 16:30'!
nativeTimeOfDayFromChronosTimeOfDay: aChronosTimeOfDay
	"Answer the native 'TimeOfDay' analog that semantically corresponds as closely as possible to <aChronosTimeOfDay>"

	"{ChronosSystemFacade current  nativeTimeOfDayFromChronosTimeOfDay: Gregorian clock timeOfDayNow}"

	^Time fromSeconds: aChronosTimeOfDay secondsSinceStartOfDay! !

!ChronosGemstoneSystemFacade methodsFor: 'value conversion' stamp: 'PaulDeBruicker 03/06/2011 16:30'!
nativeTimeZoneFromChronosTimeZone: aChronosTimezone
	"Answer the native time zoneanalog that semantically corresponds as closely as possible to <aChronosTimezone>. If there is no such value, or if Chronos is the native date/time library of the host environment, answer <aChronosTimezone>."

	| boundTZ |
	boundTZ := aChronosTimezone now timeZone. "We assume that the offset and abbreviation in effect as of the current moment are probably the best ones to use."
	^TimeZone 
		offset: (self navitiveDurationFromChronosDuration: boundTZ offsetFromUT) 
		name: self systemTimeZone name asString
		abbreviation: boundTZ commonAbbreviation! !

!ChronosGemstoneSystemFacade methodsFor: 'value conversion' stamp: 'PaulDeBruicker 03/06/2011 16:30'!
navitiveDurationFromChronosDuration: aScientificDuration 
	"Answer a ScientificDuration that semantically corresponds as closely as possible to <aNativeDuration>."

	"ChronosSystemFacade current navitiveDurationFromChronosDuration: (DurationFactory hours: -8)"

	^Duration seconds: aScientificDuration asSeconds nanoSeconds: aScientificDuration nanosecondsSinceSecond! !

!ChronosGemstoneSystemFacade methodsFor: 'instance creation' stamp: 'PaulDeBruicker 03/06/2011 16:35'!
newSystemClock
	^ChronosGemstoneSystemClock new! !

!ChronosGemstoneSystemFacade methodsFor: 'private-system timezone' stamp: 'PaulDeBruicker 03/06/2011 16:30'!
setBaseSystemTimeZoneFromChronosSystemTimeZone
	"Set the base system's timezone (that defines its local time) to one which is as congruent as possible to the receiver's <systemTimeZone>.  This operation MUST NOT affect how time is reported by Chronos. If the Smalltalk execution environment has its own support for timezones (apart from either Chronos or the operating system,) then this operation should affect how the current Smalltalk execution environment reports time using its native date-time facilities.  Ideally, this operation should also affect the reporting of time by the host operating system (at least for the current user, but probably not for all users.) 

	NOTE: Some systems keep time in UT, and report local time by converting from UT according to the specifications of the current timezone.  Other systems keep time in local terms, and convert to UT when needful according to the specifications of the current timezone.  In the former case, this operation optimally should change the local time-of-day natively reported by the Smalltalk execution environment.  In the latter case, it should change the UT time-of-day natively reported by the Smalltalk execution environment."

	self isChronologyLibraryResident ifFalse: [^self "TimeZone not present pre 3.7, so do nothing for now"].

	"The following logic only works for time zones with no DST transitions.  There is no other option, since Chronology.TimeZone doesn't do DST transitions--an issue which is someone else's problem to solve, not mine..."

	DateAndTime localTimeZone: (self nativeTimeZoneFromChronosTimeZone: self systemTimeZone)! !

!ChronosGemstoneSystemFacade methodsFor: 'private-system timezone' stamp: 'PaulDeBruicker 03/06/2011 16:30'!
setSystemTimeZoneToAnalogOfNativeSystemTimeZone
	"If it's possible to determine that the host system is using an Olson timezone to define its local time, and if it's also possible to determine the Olson key of that timezone (to the desired degreee of certainty and precision,) then set the receiver's <systemTimeZone> to a Chronos VariableOffsetTimezone with the same key and answer true.

	Otherwise, construct a ChronosTimezone that emulates the behavior of the host system's timezone as closely as possible (i.e., serves a reasonably close analog of the host system's timezone,) set the ChronosTimezone so constructed as the <systemTimeZone> and answer true. However, if it's not possible to get sufficient information about the host system's timezone to construct a ChronosTimezone that will serve as a reasonable analog, then do nothing and answer false."

	"{ChronosSystemFacade current setSystemTimeZoneToAnalogOfNativeSystemTimeZone}"

	| success |
	success := super setSystemTimeZoneToAnalogOfNativeSystemTimeZone.
	success ifTrue: [^true].
	^self isChronologyLibraryResident
		ifTrue: 
			[self setSystemTimeZone:  (self chronosTimezoneFromNativeTimeZone: DateAndTime localTimeZone).
			true]
		ifFalse: [false]! !

!ChronosGemstoneSystemFacade methodsFor: 'private-system timezone' stamp: 'PaulDeBruicker 03/06/2011 16:30'!
setSystemTimeZoneToOlsonTimeZoneMostCongruentToNativeSystem
	"Determine the key of the Olson timezone that is the most congruent to the host system's timezone, set the <systemTimeZone> to the ChronosTimezone with that key, and answer true. However, if it is not possible to obtain enough information about the host system's timezone to determine the Olson timezone key of the Olson timezone that would be reasonably congruent to the host system's timezone, then do nothing and answer false."

	| success nativeTimeZone timeZoneKey |
	success := super setSystemTimeZoneToOlsonTimeZoneMostCongruentToNativeSystem.
	success ifTrue: [^true].
	self isChronologyLibraryResident ifFalse: [^false].

	"So, we have to use the Chronology TimeZone information. We'll attempt to map first the abbreviation then the 'offset' to an Olson timezone key using the default TimeZoneLocalizationPolicy."

	nativeTimeZone := DateAndTime localTimeZone.
	timeZoneKey := 
		TimeZoneLocalizationPolicy default 
			defaultTimeZoneKeyWithCommonAbbreviation: nativeTimeZone abbreviation
			dstPreferred: true 
			ifAbsent:
				[TimeZoneLocalizationPolicy default 
					defaultTimeZoneKeyForStandardTimeOffsetSecondsFromUT: nativeTimeZone offset asSeconds
					dstPreferred: false
					ifAbsent: []].
	^timeZoneKey == nil
		ifTrue: [false]
		ifFalse:
			[self setSystemTimeZone:  (VariableOffsetTimezone at: timeZoneKey ifAbsent: [^false]).
			true]
! !
